Package: clue
Version: 0.3-68
Encoding: UTF-8
Title: Cluster Ensembles
Description: CLUster Ensembles.
Authors@R: c(person("Kurt", "Hornik", role = c("aut", "cre"),
                    email = "Kurt.Hornik@R-project.org",
		    comment = c(ORCID = "0000-0003-4198-9911")),
             person("Walter", "Böhm", role = "ctb"))
License: GPL-2
Depends: R (>= 3.2.0)
Imports: stats, cluster, graphics, methods
Suggests: e1071, lpSolve (>= 5.5.7), quadprog (>= 1.4-8), relations
Enhances: RWeka, ape, cba, cclust, flexclust, flexmix, kernlab, mclust,
        movMF, modeltools
NeedsCompilation: yes
Packaged: 2026-03-26 14:54:57 UTC; hornik
Author: Kurt Hornik [aut, cre] (ORCID: <https://orcid.org/0000-0003-4198-9911>),
  Walter Böhm [ctb]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2026-03-26 15:56:36 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:36:23 UTC; unix
Archs: clue.so.dSYM
