Package: clue
Version: 0.3-65
Encoding: UTF-8
Title: Cluster Ensembles
Description: CLUster Ensembles.
Authors@R: c(person("Kurt", "Hornik", role = c("aut", "cre"),
                    email = "Kurt.Hornik@R-project.org",
		    comment = c(ORCID = "0000-0003-4198-9911")),
             person("Walter", "Böhm", role = "ctb"))
License: GPL-2
Depends: R (>= 3.2.0)
Imports: stats, cluster, graphics, methods
Suggests: e1071, lpSolve (>= 5.5.7), quadprog (>= 1.4-8), relations
Enhances: RWeka, ape, cba, cclust, flexclust, flexmix, kernlab, mclust,
        movMF, modeltools
NeedsCompilation: yes
Packaged: 2023-09-23 11:02:49 UTC; hornik
Author: Kurt Hornik [aut, cre] (<https://orcid.org/0000-0003-4198-9911>),
  Walter Böhm [ctb]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2023-09-23 11:25:57 UTC
Built: R 4.2.0; aarch64-apple-darwin20; 2023-09-24 20:56:43 UTC; unix
Archs: clue.so.dSYM
