bibentry(bibtype = "Book",
         key = "Arabie+Carroll+DeSarbo:1987",
         author = c(person(given = "Phipps",
                           family = "Arabie"),
                    person(given = c("J.", "Douglas"),
                           family = "Carroll"),
                    person(given = c("Wayne", "S."),
                           family = "DeSarbo")),
         title = "Three-way Scaling and Clustering",
         publisher = "Sage Publications",
         year = "1987",
         series = "Quantitative Applications in the Social Sciences",
         address = "Newbury Park, CA",
         doi = "10.4135/9781412986359")

bibentry(bibtype = "Book",
         key = "Barthélemy+Guénoche:1991",
         author = c(person(given = "Jean-Pierre",
                           family = "Barthélemy"),
                    person(given = "Alain",
                           family = "Guénoche")),
         title = "Trees and Proximity Representations",
         publisher = "Wiley",
         year = "1991",
         series = "Wiley Series in Discrete Mathematics and Optimization",
         address = "Chicester",
         isbn = "978-0471922636")

bibentry(bibtype = "Article",
         key = "Barthélemy+Leclerc+Monjardet:1986",
         title = "On the Use of Ordered Sets in Problems of Comparison and Consensus of Classifications",
         volume = "3",
         .issn = "1432-1343",
         doi = "10.1007/bf01894188",
         number = "2",
         journal = "Journal of Classification",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Jean-Pierre",
                           family = "Barthélemy"),
                    person(given = "Bruno",
                           family = "Leclerc"),
                    person(given = "Bernard",
                           family = "Monjardet")),
         year = "1986",
         month = "sep",
         pages = "187--224")

bibentry(bibtype = "Book",
         key = "Bezdek:1981",
         author = person(given = c("James", "C."),
                         family = "Bezdek"),
         title = "Pattern Recognition with Fuzzy Objective Function Algorithms",
         publisher = "Plenum",
         year = "1981",
         address = "New York",
         doi = "10.1007/978-1-4757-0450-1")

bibentry(bibtype = "InCollection",
         key = "Boorman+Arabie:1972",
         author = c(person(given = c("Scott", "A."),
                           family = "Boorman"),
                    person(given = "Phipps",
                           family = "Arabie")),
         title = "Structural Measures and the Method of Sorting",
         booktitle = "Multidimensional Scaling: Theory and Applications in the Behavioral Sciences, 1: Theory",
         publisher = "Seminar Press",
         year = "1972",
         editor = c(person(given = c("R.", "N."),
                           family = "Shepard"),
                    person(given = c("A.", "K."),
                           family = "Romney"),
                    person(given = c("S.", "B."),
                           family = "Nerlove")),
         pages = "225--249",
         address = "New York")

bibentry(bibtype = "Article",
         key = "Boorman+Olivier:1973",
         title = "Metrics on Spaces of Finite Trees",
         volume = "10",
         .issn = "0022-2496",
         doi = "10.1016/0022-2496(73)90003-5",
         number = "1",
         journal = "Journal of Mathematical Psychology",
         publisher = "Elsevier BV",
         author = c(person(given = c("Scott", "A."),
                           family = "Boorman"),
                    person(given = c("Donald", "C."),
                           family = "Olivier")),
         year = "1973",
         month = "feb",
         pages = "26--59")

bibentry(bibtype = "InCollection",
         key = "Carroll+Pruzansky:1980",
         author = c(person(given = c("J.", "D."),
                           family = "Carroll"),
                    person(given = "S.",
                           family = "Pruzansky")),
         title = "Discrete and Hybrid Scaling Models",
         booktitle = "Similarity and Choice",
         publisher = "Huber",
         year = "1980",
         editor = c(person(given = c("E.", "D."),
                           family = "Lantermann"),
                    person(given = "H.",
                           family = "Feger")),
         address = "Bern, Switzerland")

bibentry(bibtype = "Article",
         key = "Charon+Denoeud+Guénoche:2006",
         title = "Maximum Transfer Distance Between Partitions",
         volume = "23",
         .issn = "1432-1343",
         doi = "10.1007/s00357-006-0006-2",
         number = "1",
         journal = "Journal of Classification",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Irene",
                           family = "Charon"),
                    person(given = "Lucile",
                           family = "Denoeud"),
                    person(given = "Alain",
                           family = "Guénoche"),
                    person(given = "Olivier",
                           family = "Hudry")),
         year = "2006",
         month = "jun",
         pages = "103--121")

bibentry(bibtype = "Article",
         key = "De_Soete:1983",
         title = "A Least Squares Algorithm for Fitting Additive Trees to Proximity Data",
         volume = "48",
         .issn = "1860-0980",
         doi = "10.1007/bf02293884",
         number = "4",
         journal = "Psychometrika",
         publisher = "Cambridge University Press (CUP)",
         author = person(given = "Geert",
                         family = "De Soete"),
         year = "1983",
         month = "dec",
         pages = "621--626")

bibentry(bibtype = "Article",
         key = "Day:1981",
         title = "The Complexity of Computing Metric Distances Between Partitions",
         volume = "1",
         .issn = "0165-4896",
         doi = "10.1016/0165-4896(81)90042-1",
         number = "3",
         journal = "Mathematical Social Sciences",
         publisher = "Elsevier BV",
         author = person(given = c("William", "H.", "E."),
                         family = "Day"),
         year = "1981",
         month = "may",
         pages = "269--287")

bibentry(bibtype = "Article",
         key = "De_Soete:1984a",
         title = "A Least Squares Algorithm for Fitting an Ultrametric Tree to a Dissimilarity Matrix",
         volume = "2",
         .issn = "0167-8655",
         doi = "10.1016/0167-8655(84)90036-9",
         number = "3",
         journal = "Pattern Recognition Letters",
         publisher = "Elsevier BV",
         author = person(given = "Geert",
                         family = "De Soete"),
         year = "1984",
         month = "mar",
         pages = "133--137")

bibentry(bibtype = "Article",
         key = "De_Soete:1984b",
         title = "Ultrametric Tree Representations of Incomplete Dissimilarity Data",
         volume = "1",
         .issn = "1432-1343",
         doi = "10.1007/bf01890124",
         number = "1",
         journal = "Journal of Classification",
         publisher = "Springer Science and Business Media LLC",
         author = person(given = "Geert",
                         family = "De Soete"),
         year = "1984",
         month = "dec",
         pages = "235--242")

bibentry(bibtype = "Article",
         key = "Dimitriadou+Weingessel+Hornik:2002",
         title = "A Combination Scheme for Fuzzy Clustering",
         volume = "16",
         .issn = "1793-6381",
         doi = "10.1142/s0218001402002052",
         number = "07",
         journal = "International Journal of Pattern Recognition and Artificial Intelligence",
         publisher = "World Scientific Pub Co Pte Lt",
         author = c(person(given = "Evgenia",
                           family = "Dimitriadou"),
                    person(given = "Andreas",
                           family = "Weingessel"),
                    person(given = "Kurt",
                           family = "Hornik")),
         year = "2002",
         month = "nov",
         pages = "901--912")

bibentry(bibtype = "Article",
         key = "Dudoit+Fridlyand:2003",
         title = "Bagging to Improve the Accuracy of a Clustering Procedure",
         volume = "19",
         .issn = "1367-4803",
         doi = "10.1093/bioinformatics/btg038",
         number = "9",
         journal = "Bioinformatics",
         publisher = "Oxford University Press (OUP)",
         author = c(person(given = "Sandrine",
                           family = "Dudoit"),
                    person(given = "Jane",
                           family = "Fridlyand")),
         year = "2003",
         month = "jun",
         pages = "1090--1099")

bibentry(bibtype = "Book",
         key = "Fiacco+McCormick:1968",
         author = c(person(given = c("Anthony", "V."),
                           family = "Fiacco"),
                    person(given = c("Garth", "P."),
                           family = "McCormick")),
         title = "Nonlinear Programming: Sequential Unconstrained Minimization Techniques",
         publisher = "John Wiley & Sons",
         year = "1968",
         address = "New York")

bibentry(bibtype = "Article",
         key = "Fowlkes+Mallows:1983",
         title = "A Method for Comparing Two Hierarchical Clusterings",
         volume = "78",
         .issn = "1537-274X",
         doi = "10.1080/01621459.1983.10478008",
         number = "383",
         journal = "Journal of the American Statistical Association",
         publisher = "Informa UK Limited",
         author = c(person(given = c("E.", "B."),
                           family = "Fowlkes"),
                    person(given = c("C.", "L."),
                           family = "Mallows")),
         year = "1983",
         month = "sep",
         pages = "553--569")

bibentry(bibtype = "Article",
         key = "Gaul+Schader:1988",
         title = "Clusterwise Aggregation of Relations",
         volume = "4",
         .issn = "1099-0747",
         doi = "10.1002/asm.3150040406",
         number = "4",
         journal = "Applied Stochastic Models and Data Analysis",
         publisher = "Wiley",
         author = c(person(given = "Wolfgang",
                           family = "Gaul"),
                    person(given = "Martin",
                           family = "Schader")),
         year = "1988",
         month = "dec",
         pages = "273--282")

bibentry(bibtype = "Article",
         key = "Gusfield:2002",
         title = "Partition-Distance: A Problem and Class of Perfect Graphs Arising in Clustering",
         volume = "82",
         .issn = "0020-0190",
         doi = "10.1016/s0020-0190(01)00263-0",
         number = "3",
         journal = "Information Processing Letters",
         publisher = "Elsevier BV",
         author = person(given = "Dan",
                         family = "Gusfield"),
         year = "2002",
         month = "may",
         pages = "159--164")

bibentry(bibtype = "Article",
         key = "Gordon+Vichi:1998",
         title = "Partitions of Partitions",
         volume = "15",
         .issn = "1432-1343",
         doi = "10.1007/s003579900034",
         number = "2",
         journal = "Journal of Classification",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = c("Allan", "Drummond"),
                           family = "Gordon"),
                    person(given = "Maurizio",
                           family = "Vichi")),
         year = "1998",
         month = "feb",
         pages = "265--285")

bibentry(bibtype = "Article",
         key = "Gordon+Vichi:2001",
         title = "Fuzzy Partition Models for Fitting a Set of Partitions",
         volume = "66",
         .issn = "1860-0980",
         doi = "10.1007/bf02294837",
         number = "2",
         journal = "Psychometrika",
         publisher = "Cambridge University Press (CUP)",
         author = c(person(given = c("Allan", "Drummond"),
                           family = "Gordon"),
                    person(given = "Maurizio",
                           family = "Vichi")),
         year = "2001",
         month = "jun",
         pages = "229--247")

bibentry(bibtype = "Article",
         key = "Hubert+Arabie:1985",
         title = "Comparing Partitions",
         volume = "2",
         .issn = "1432-1343",
         doi = "10.1007/bf01908075",
         number = "1",
         journal = "Journal of Classification",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Lawrence",
                           family = "Hubert"),
                    person(given = "Phipps",
                           family = "Arabie")),
         year = "1985",
         month = "dec",
         pages = "193--218")

bibentry(bibtype = "Article",
         key = "Hubert+Arabie:1995",
         title = "Iterative Projection Strategies for the Least-Squares Fitting of Tree Structures to Proximity Data",
         volume = "48",
         .issn = "2044-8317",
         doi = "10.1111/j.2044-8317.1995.tb01065.x",
         number = "2",
         journal = "British Journal of Mathematical and Statistical Psychology",
         publisher = "Wiley",
         author = c(person(given = "Lawrence",
                           family = "Hubert"),
                    person(given = "Phipps",
                           family = "Arabie")),
         year = "1995",
         month = "nov",
         pages = "281--317")

bibentry(bibtype = "Book",
         key = "Hubert+Arabie+Meulman:2006",
         author = c(person(given = "Lawrence",
                           family = "Hubert"),
                    person(given = "Phipps",
                           family = "Arabie"),
                    person(given = "Jacqueline",
                           family = "Meulman")),
         title = "The Structural Representation of Proximity Matrices with {MATLAB}",
         publisher = "SIAM",
         year = "2006",
         series = "ASA-SIAM Series on Statistics and Applied Mathematics",
         address = "Philadelphia, PA",
         doi = "10.1137/1.9780898718355")

bibentry(bibtype = "Book",
         key = "Jardine+Sibson:1971",
         author = c(person(given = "Nicholas",
                           family = "Jardine"),
                    person(given = "Robin",
                           family = "Sibson")),
         title = "Mathematical Taxonomy",
         publisher = "Wiley",
         year = "1971",
         address = "London")

bibentry(bibtype = "Article",
         key = "Katz+Powell:1953",
         title = "A Proposed Index of the Conformity of One Sociometric Measurement to Another",
         volume = "18",
         .issn = "1860-0980",
         doi = "10.1007/bf02289063",
         number = "3",
         journal = "Psychometrika",
         publisher = "Cambridge University Press (CUP)",
         author = c(person(given = "Leo",
                           family = "Katz"),
                    person(given = c("James", "H."),
                           family = "Powell")),
         year = "1953",
         month = "sep",
         pages = "249--256")

bibentry(bibtype = "Book",
         key = "Kaufman+Rousseeuw:1990",
         author = c(person(given = "Leonard",
                           family = "Kaufman"),
                    person(given = c("Peter", "J."),
                           family = "Rousseeuw")),
         title = "Finding Groups in Data: An Introduction to Cluster Analysis",
         publisher = "Wiley",
         year = "1990",
         address = "New York",
         doi = "10.1002/9780470316801")

bibentry(bibtype = "Article",
         key = "Křivánek+Morávek:1986",
         title = "{NP}-hard Problems in Hierarchical-Tree Clustering",
         volume = "23",
         .issn = "1432-0525",
         doi = "10.1007/bf00289116",
         number = "3",
         journal = "Acta Informatica",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Mirko",
                           family = "Křivánek"),
                    person(given = "Jaroslav",
                           family = "Morávek")),
         year = "1986",
         month = "jun",
         pages = "311--323")

bibentry(bibtype = "Misc",
         key = "Leisch:1999",
         doi = "10.57938/9B129F95-B53B-44CE-A129-5B7A1168D832",
         author = person(given = "Friedrich",
                         family = "Leisch"),
         language = "en",
         title = "Bagged Clustering",
         publisher = "Vienna University of Economics and Business",
         year = "1999")

bibentry(bibtype = "Article",
         key = "Margush+McMorris:1981",
         title = "Consensus $n$-Trees",
         volume = "43",
         .issn = "1522-9602",
         doi = "10.1007/bf02459446",
         number = "2",
         journal = "Bulletin of Mathematical Biology",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "T.",
                           family = "Margush"),
                    person(given = c("F.", "R."),
                           family = "McMorris")),
         year = "1981",
         month = "mar",
         pages = "239--244")

bibentry(bibtype = "InProceedings",
         key = "Meila:2003",
         author = person(given = "Marina",
                         family = "Meilă"),
         editor = c(person(given = "Bernhard",
                           family = "Schölkopf"),
                    person(given = c("Manfred", "K."),
                           family = "Warmuth")),
         title = "Comparing Clusterings by the Variation of Information",
         booktitle = "Learning Theory and Kernel Machines",
         year = "2003",
         publisher = "Springer Berlin Heidelberg",
         address = "Berlin, Heidelberg",
         pages = "173--187",
         isbn = "978-3-540-45167-9",
         doi = "10.1007/978-3-540-45167-9_14")

bibentry(bibtype = "Article",
         key = "Mérigot+Durbec+Gaertner:2010",
         title = "On Goodness-of-Fit Measure for Dendrogram‐Based Analyses",
         volume = "91",
         .issn = "1939-9170",
         doi = "10.1890/09-1387.1",
         number = "6",
         journal = "Ecology",
         publisher = "Wiley",
         author = c(person(given = "Bastien",
                           family = "Mérigot"),
                    person(given = "Jean-Pierre",
                           family = "Durbec"),
                    person(given = "Jean-Claude",
                           family = "Gaertner")),
         year = "2010",
         month = "jun",
         pages = "1850--1859")

bibentry(bibtype = "Article",
         key = "Miller+Nicely:1955",
         title = "An Analysis of Perceptual Confusions Among Some English Consonants",
         volume = "27",
         .issn = "1520-8524",
         doi = "10.1121/1.1907526",
         number = "2",
         journal = "The Journal of the Acoustical Society of America",
         publisher = "Acoustical Society of America (ASA)",
         author = c(person(given = c("George", "A."),
                           family = "Miller"),
                    person(given = c("Patricia", "E."),
                           family = "Nicely")),
         year = "1955",
         month = "mar",
         pages = "338--352")

bibentry(bibtype = "Book",
         key = "Papadimitriou+Steiglitz:1982",
         author = c(person(given = c("Christos", "H."),
                           family = "Papadimitriou"),
                    person(given = "Kenneth",
                           family = "Steiglitz")),
         title = "Combinatorial Optimization: Algorithms and Complexity",
         publisher = "Prentice Hall",
         year = "1982",
         address = "Englewood Cliffs")

bibentry(bibtype = "Article",
         key = "Rajski:1961",
         title = "A Metric Space of Discrete Probability Distributions",
         volume = "4",
         .issn = "0019-9958",
         doi = "10.1016/s0019-9958(61)80055-7",
         number = "4",
         journal = "Information and Control",
         publisher = "Elsevier BV",
         author = person(given = "C.",
                         family = "Rajski"),
         year = "1961",
         month = "dec",
         pages = "371--377")

bibentry(bibtype = "Article",
         key = "Rand:1971",
         title = "Objective Criteria for the Evaluation of Clustering Methods",
         volume = "66",
         .issn = "1537-274X",
         doi = "10.1080/01621459.1971.10482356",
         number = "336",
         journal = "Journal of the American Statistical Association",
         publisher = "Informa UK Limited",
         author = person(given = c("William", "M."),
                         family = "Rand"),
         year = "1971",
         month = "dec",
         pages = "846--850")

bibentry(bibtype = "InCollection",
         key = "Rosenberg:1982",
         author = person(given = "Seymour",
                         family = "Rosenberg"),
         title = "The Method of Sorting in Multivariate Research with Applications Selected from Cognitive Psychology and Person Perception",
         booktitle = "Multivariate Applications in the Social Sciences",
         publisher = "Erlbaum",
         year = "1982",
         editor = c(person(given = "N.",
                           family = "Hirschberg"),
                    person(given = c("L.", "G."),
                           family = "Humphreys")),
         pages = "117--142",
         address = "Hillsdale, NJ",
         doi = "10.4324/9781315802831")

bibentry(bibtype = "Article",
         key = "Rosenberg+Kim:1975",
         title = "The Method of Sorting as a Data-Gathering Procedure in Multivariate Research",
         volume = "10",
         .issn = "1532-7906",
         doi = "10.1207/s15327906mbr1004_7",
         number = "4",
         journal = "Multivariate Behavioral Research",
         publisher = "Informa UK Limited",
         author = c(person(given = "Seymour",
                           family = "Rosenberg"),
                    person(given = c("Moonja", "Park"),
                           family = "Kim")),
         year = "1975",
         month = "oct",
         pages = "489--502")

bibentry(bibtype = "InCollection",
         key = "Roux:1988",
         title = "Techniques of Approximation for Building Two Tree Structures",
         editor = c(person(given = "Chikio",
                           family = "Hayashi"),
                    person(given = "Michel",
                           family = "Jambu"),
                    person(given = "Edwin",
                           family = "Diday"),
                    person(given = "Noboru",
                           family = "Ohsumi")),
         booktitle = "Recent Developments in Clustering and Data Analysis",
         publisher = "Academic Press",
         pages = "151--170",
         year = "1988",
         isbn = "978-0-12-215485-0",
         doi = "10.1016/B978-0-12-215485-0.50017-X",
         author = person(given = "M.",
                         family = "Roux"))

bibentry(bibtype = "Article",
         key = "Rubin:1967",
         title = "Optimal Classification into Groups: An Approach for Solving The Taxonomy Problem",
         volume = "15",
         .issn = "0022-5193",
         doi = "10.1016/0022-5193(67)90046-x",
         number = "1",
         journal = "Journal of Theoretical Biology",
         publisher = "Elsevier BV",
         author = person(given = "Jerrold",
                         family = "Rubin"),
         year = "1967",
         month = "apr",
         pages = "103--144")

bibentry(bibtype = "Article",
         key = "Sato+Sato:1994",
         title = "On a Multicriteria Fuzzy Clustering Method for 3-Way Data",
         volume = "02",
         .issn = "1793-6411",
         doi = "10.1142/s0218488594000122",
         number = "02",
         journal = "International Journal of Uncertainty, Fuzziness and Knowledge-Based Systems",
         publisher = "World Scientific Pub Co Pte Lt",
         author = c(person(given = "Mika",
                           family = "Sato"),
                    person(given = "Yoshiharu",
                           family = "Sato")),
         year = "1994",
         month = "jun",
         pages = "127--142")

bibentry(bibtype = "Article",
         key = "Smith:2001",
         author = person(given = c("Thomas", "J."),
                         family = "Smith"),
         title = "Constructing Ultrametric and Additive Trees Based on the $L_1$ Norm",
         journal = "Journal of Classification",
         year = "2001",
         volume = "18",
         number = "2",
         pages = "185--207",
         doi = "10.1007/s00357-001-0015-0")

bibentry(bibtype = "Article",
         key = "Strehl+Ghosh:2002",
         author = c(person(given = "Alexander",
                           family = "Strehl"),
                    person(given = "Joydeep",
                           family = "Ghosh")),
         title = "Cluster Ensembles --- A Knowledge Reuse Framework for Combining Multiple Partition",
         journal = "Journal of Machine Learning Research",
         year = "2002",
         volume = "3",
         pages = "583--617",
         url = "https://jmlr.org/papers/v3/strehl02a.html")

bibentry(bibtype = "Article",
         key = "Tibshirani+Walther:2005",
         title = "Cluster Validation by Prediction Strength",
         volume = "14",
         .issn = "1537-2715",
         doi = "10.1198/106186005x59243",
         number = "3",
         journal = "Journal of Computational and Graphical Statistics",
         publisher = "Informa UK Limited",
         author = c(person(given = "Robert",
                           family = "Tibshirani"),
                    person(given = "Guenther",
                           family = "Walther")),
         year = "2005",
         month = "sep",
         pages = "511--528")

bibentry(bibtype = "InProceedings",
         key = "Zhou+Li+Zha:2005",
         series = "ICML ’05",
         title = "A New {Mallows} Distance Based Metric for Comparing Clusterings",
         doi = "10.1145/1102351.1102481",
         booktitle = "Proceedings of the 22nd international conference on Machine learning -- ICML ’05",
         publisher = "ACM Press",
         author = c(person(given = "Ding",
                           family = "Zhou"),
                    person(given = "Jia",
                           family = "Li"),                    
                    person(given = "Hongyuan",
                           family = "Zha")),
         year = "2005",
         pages = "1028--1035",
         collection = "ICML ’05")

